@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     flaskext.mail
     ~~~~~~~~~~~~~
@@ -10,22 +9,82 @@
 """
 from __future__ import with_statement
 
-__version__ = '0.7.3'
+__version__ = '0.7.5'
 
 import blinker
 import smtplib
 import socket
 import time
 
+from email import charset
 from email.encoders import encode_base64
 from email.mime.base import MIMEBase
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
-from email.utils import formatdate, make_msgid
+from email.header import Header
+from email.utils import formatdate, formataddr, make_msgid, parseaddr
 from contextlib import contextmanager
 
 from flask import current_app
 
+charset.add_charset('utf-8', charset.SHORTEST, None, 'utf-8')
+
+class FlaskMailUnicodeDecodeError(UnicodeDecodeError):
+    def __init__(self, obj, *args):
+        self.obj = obj
+        UnicodeDecodeError.__init__(self, *args)
+
+    def __str__(self):
+        original = UnicodeDecodeError.__str__(self)
+
+        return '%s. You passed in %r (%s)' % (original, self.obj, type(self.obj))
+
+def force_text(s, encoding='utf-8', errors='stricts'):
+    if isinstance(s, unicode):
+        return s
+
+    try:
+        if not isinstance(s, basestring):
+            if hasattr(s, '__unicode__'):
+                s = s.__unicode__()
+            else:
+                s = unicode(bytes(s), encoding, errors)
+        else:
+            s = s.decode(encoding, errors)
+    except UnicodeDecodeError as e:
+        if not isinstance(s, Exception):
+            raise FlaskMailUnicodeDecodeError(s, *e.args)
+        else:
+            s = ' '.join([force_text(arg, encoding, errors) for arg in s])
+
+    return s
+
+def sanitize_address(addr, encoding='utf-8'):
+    if isinstance(addr, basestring):
+        addr = parseaddr(force_text(addr))
+
+    nm, addr = addr
+
+    # This try-except clause is needed on Python 3 < 3.2.4
+    # http://bugs.python.org/issue14291
+
+    try:
+        nm = Header(nm, encoding).encode()
+    except UnicodeEncodeError:
+        nm = Header(nm, 'utf-8').encode()
+
+    try:
+        addr.encode('ascii')
+    except UnicodeEncodeError:  # IDN
+        if '@' in addr:
+            localpart, domain = addr.split('@', 1)
+            localpart = str(Header(localpart, encoding))
+            domain = domain.encode('idna').decode('ascii')
+            addr = '@'.join([localpart, domain])
+        else:
+            addr = Header(addr, encoding).encode()
+
+    return formataddr((nm, addr))
 
 class Connection(object):
 
@@ -114,7 +173,6 @@ class Connection(object):
 
         self.send(Message(*args, **kwargs))
 
-
 class BadHeaderError(Exception):
     pass
 
@@ -141,7 +199,6 @@ class Attachment(object):
         self.disposition = disposition or 'attachment'
         self.headers = headers or {}
 
-
 class Message(object):
 
     """
@@ -182,7 +239,7 @@ class Message(object):
             sender = "%s <%s>" % sender
 
         self.subject = subject
-        self.sender = sender
+        self.sender = sanitize_address(sender)
         self.body = body
         self.html = html
         self.date = date
@@ -190,14 +247,14 @@ class Message(object):
         self.charset = charset
         self.extra_headers = extra_headers
 
-        self.cc = cc
-        self.bcc = bcc
-        self.reply_to = reply_to
+        self.cc = sanitize_address(cc) if cc is not None else None
+        self.bcc = sanitize_address(bcc) if bcc is not None else None
+        self.reply_to = sanitize_address(reply_to) if reply_to is not None else None
 
         if recipients is None:
             recipients = []
 
-        self.recipients = list(recipients)
+        self.recipients = list([sanitize_address(recipient) for recipient in recipients])
 
         if attachments is None:
             attachments = []
@@ -230,9 +287,11 @@ class Message(object):
             msg.attach(self._mimetext(self.body))
         else:
             # Anything else
-            msg = MIMEMultipart('alternative')
-            msg.attach(self._mimetext(self.body, 'plain'))
-            msg.attach(self._mimetext(self.html, 'html'))
+            msg = MIMEMultipart()
+            alternative = MIMEMultipart('alternative')
+            alternative.attach(self._mimetext(self.body, 'plain'))
+            alternative.attach(self._mimetext(self.html, 'html'))
+            msg.attach(alternative)
 
         msg['Subject'] = self.subject
         msg['From'] = self.sender
@@ -242,9 +301,6 @@ class Message(object):
         # see RFC 5322 section 3.6.4.
         msg['Message-ID'] = self.msgId
 
-        if self.bcc:
-            msg['Bcc'] = ', '.join(self.bcc)
-
         if self.cc:
             msg['Cc'] = ', '.join(self.cc)
 
@@ -297,7 +353,6 @@ class Message(object):
         """
 
         assert self.recipients, "No recipients have been added"
-        assert self.body or self.html, "No body or HTML has been set"
         assert self.sender, "No sender address has been set"
 
         if self.has_bad_headers():
@@ -366,7 +421,7 @@ class Mail(object):
         self.debug = int(app.config.get('MAIL_DEBUG', app.debug))
         self.max_emails = app.config.get('DEFAULT_MAX_EMAILS')
         self.suppress = app.config.get('MAIL_SUPPRESS_SEND', False)
-        self.fail_silently = app.config.get('MAIL_FAIL_SILENTLY', True)
+        self.fail_silently = app.config.get('MAIL_FAIL_SILENTLY', False)
 
         self.suppress = self.suppress or app.testing
         self.app = app
